@@ -24,11 +24,15 @@
 #include <config.h>
 #endif
 
+#include <gal/widgets/e-unicode.h>
+
 #include "gtkhtml.h"
+
 #include "htmlclueflow.h"
 #include "htmlcursor.h"
 #include "htmlengine.h"
 #include "htmllinktext.h"
+#include "htmlengine-edit.h"
 #include "htmlengine-edit-cut-and-paste.h"
 #include "htmlengine-edit-table.h"
 #include "htmlengine-edit-tablecell.h"
@@ -75,9 +79,11 @@ paste (GtkWidget *mi, GtkHTMLControlData *cd)
 static void
 remove_link (GtkWidget *mi, GtkHTMLControlData *cd)
 {
+	html_engine_selection_push (cd->html->engine);
 	if (!html_engine_is_selection_active (cd->html->engine))
-		html_engine_select_word (cd->html->engine);
+		html_engine_select_word_editable (cd->html->engine);
 	html_engine_insert_link (cd->html->engine, NULL, NULL);
+	html_engine_selection_pop (cd->html->engine);
 }
 
 static void
@@ -239,8 +245,13 @@ static void
 spell_add (GtkWidget *mi, GtkHTMLControlData *cd)
 {
 	HTMLEngine *e = cd->html->engine;
+	gchar *word;
 
-	spell_add_to_personal (cd->html, html_engine_get_word (e), cd);
+	word = html_engine_get_word (e);
+	if (word) {
+		spell_add_to_personal (cd->html, word, cd);
+		g_free (word);
+	}
 	html_engine_spell_check (e);
 }
 
@@ -248,8 +259,13 @@ static void
 spell_ignore (GtkWidget *mi, GtkHTMLControlData *cd)
 {
 	HTMLEngine *e = cd->html->engine;
+	gchar *word;
 
-	spell_add_to_session (cd->html, html_engine_get_word (e), cd);
+	word = html_engine_get_word (e);
+	if (word) {
+		spell_add_to_session (cd->html, word, cd);
+		g_free (word);
+	}
 	html_engine_spell_check (e);
 }
 
@@ -273,14 +289,21 @@ insert_html (GtkWidget *mi, GtkHTMLControlData *cd)
 }
 #endif
 
-#define ADD_ITEM(l,f,t) \
-		menuitem = gtk_menu_item_new_with_label (l); \
+#define ADD_ITEM_BASE(f,t) \
                 gtk_object_set_data (GTK_OBJECT (menuitem), "type", GINT_TO_POINTER (GTK_HTML_EDIT_PROPERTY_ ## t)); \
 		gtk_menu_append (GTK_MENU (menu), menuitem); \
 		gtk_widget_show (menuitem); \
 		gtk_signal_connect (GTK_OBJECT (menuitem), "activate", GTK_SIGNAL_FUNC (f), cd); \
                 (*items)++; items_sep++
 
+#define ADD_ITEM(l,f,t) \
+		menuitem = gtk_menu_item_new_with_label (l); \
+                ADD_ITEM_BASE (f,t)
+
+#define ADD_ITEM_UTF8(l,f,t) \
+                menuitem = e_utf8_gtk_menu_item_new_with_label (GTK_MENU (menu), l); \
+                ADD_ITEM_BASE (f,t)
+
 #define ADD_SEP \
         if (items_sep) { \
                 menuitem = gtk_menu_item_new (); \
@@ -332,20 +355,26 @@ prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items)
 #endif
 
 	if (!html_engine_is_selection_active (e) && obj && html_object_is_text (obj) && !html_engine_word_is_valid (e)) {
-		gchar *spell, *word, *ignore, *add;
+		gchar *spell, *word, *check_utf8, *add_utf8, *ignore_utf8, *ignore, *add;
 
 		word   = html_engine_get_word (e);
-		spell  = g_strdup_printf (_("Check '%s' spelling..."), word);
-		add    = g_strdup_printf (_("Add '%s' to dictionary"), word);
-		ignore = g_strdup_printf (_("Ignore '%s'"), word);
+		check_utf8 = e_utf8_from_locale_string (_("Check '%s' spelling..."));
+		spell  = g_strdup_printf (check_utf8, word);
+		g_free (check_utf8);
+		add_utf8 = e_utf8_from_locale_string (_("Add '%s' to dictionary"));
+		add    = g_strdup_printf (add_utf8, word);
+		g_free (add_utf8);
+		ignore_utf8 = e_utf8_from_locale_string (_("Ignore '%s'"));
+		ignore = g_strdup_printf (ignore_utf8, word);
+		g_free (ignore_utf8);
 		SUBMENU (N_("Spell checker"));
 		if (cd->has_spell_control) {
-			ADD_ITEM (spell, spell_check_cb, NONE);
+			ADD_ITEM_UTF8 (spell, spell_check_cb, NONE);
 		} else {
 			ADD_ITEM (_("Suggest word"), spell_suggest, NONE);
 		}
-		ADD_ITEM (add, spell_add, NONE);
-		ADD_ITEM (ignore, spell_ignore, NONE);
+		ADD_ITEM_UTF8 (add, spell_add, NONE);
+		ADD_ITEM_UTF8 (ignore, spell_ignore, NONE);
 		END_SUBMENU;
 
 		g_free (spell);
