@@ -35,7 +35,7 @@ find_first (HTMLEngine *e)
 	guchar c;
 
 	c = html_cursor_get_current_char (e->cursor);
-	while (c == 0 || ! isalnum ((gint) c) || c == ' ') {
+	while (c == 0 || ! g_unichar_isalnum (c) || c == ' ') {
 		if (!html_cursor_forward (e->cursor, e))
 			return FALSE;
 		c = html_cursor_get_current_char (e->cursor);
@@ -47,17 +47,15 @@ find_first (HTMLEngine *e)
 static void
 upper_lower (HTMLObject *obj, HTMLEngine *e, gpointer data)
 {
-	gboolean up = GPOINTER_TO_INT (data);
-	guchar *text;
+	if (html_object_is_text (obj)) {
+		gboolean up = GPOINTER_TO_INT (data);
+		guchar *text;
 
-	printf ("upper_lower\n");
-
-	g_assert (html_object_is_text (obj));
-
-	text = HTML_TEXT (obj)->text;
-	while (*text) {
-		*text = (up) ? toupper (*text) : tolower (*text);
-		text++;
+		text = HTML_TEXT (obj)->text;
+		while (*text) {
+			*text = (up) ? toupper (*text) : tolower (*text);
+			text++;
+		}
 	}
 }
 
@@ -65,19 +63,25 @@ void
 html_engine_capitalize_word (HTMLEngine *e)
 {
 	if (find_first (e)) {
+		guchar c;
+
 		html_undo_level_begin (e->undo, "Capitalize word", "Revert word capitalize");
 		html_engine_set_mark (e);
 		html_cursor_forward (e->cursor, e);
 		html_engine_cut_and_paste (e, "up 1st", "revert up 1st",
 					   upper_lower, GINT_TO_POINTER (TRUE));
 		html_engine_disable_selection (e);
-		html_engine_set_mark (e);
-		html_engine_forward_word (e);
-		html_engine_cut_and_paste (e, "down rest", "revert down rest",
-					   upper_lower, GINT_TO_POINTER (FALSE));
-		html_engine_disable_selection (e);
+
+		c = html_cursor_get_current_char (e->cursor);
+		if (g_unichar_isalnum (c)) {
+			html_engine_set_mark (e);
+			html_engine_forward_word (e);
+			html_engine_cut_and_paste (e, "down rest", "revert down rest",
+						   upper_lower, GINT_TO_POINTER (FALSE));
+			html_engine_disable_selection (e);
+		}
 		html_undo_level_end (e->undo);
-	}
+		}
 }
 
 void
