@@ -37,8 +37,9 @@ typedef enum {
 	HTML_CHANGE_NONE = 0,
 	HTML_CHANGE_MIN_WIDTH = 1 << 0,
 	HTML_CHANGE_PREF_WIDTH = 1 << 1,
-	HTML_CHANGE_WORD_WIDTH = 1 << 2,
-	HTML_CHANGE_ALL_CALC = ((HTML_CHANGE_PREF_WIDTH << 1) - 1),
+	HTML_CHANGE_SIZE = 1 << 2,
+	HTML_CHANGE_WORD_WIDTH = 1 << 3,
+	HTML_CHANGE_ALL_CALC = ((HTML_CHANGE_SIZE << 1) - 1),
 	HTML_CHANGE_ALL = ((HTML_CHANGE_WORD_WIDTH << 1) - 1)
 } HTMLChangeFlags;
 
