@@ -21,11 +21,24 @@
 
 #include <gtk/gtkselection.h>
 #include "htmlcursor.h"
+#include "htmlentity.h"
 #include "htmlinterval.h"
 #include "htmlselection.h"
 #include "htmlengine-edit.h"
 #include "htmlengine-edit-selection-updater.h"
 
+guint32
+html_selection_current_time (void)
+{
+	GdkEvent *event;
+
+	event = gtk_get_current_event ();
+	if (event != NULL)
+		return gdk_event_get_time (event);
+
+	return GDK_CURRENT_TIME;
+}
+
 void
 html_engine_select_interval (HTMLEngine *e, HTMLInterval *i)
 {
@@ -37,7 +50,8 @@ html_engine_select_interval (HTMLEngine *e, HTMLInterval *i)
 		e->selection = i;
 		html_interval_select (e->selection, e);
 	}
-	html_engine_activate_selection (e, GDK_CURRENT_TIME);
+
+	html_engine_activate_selection (e, html_selection_current_time ());
 }
 
 void
@@ -78,6 +92,11 @@ html_engine_clear_selection (HTMLEngine *e)
 		html_interval_destroy (e->selection);
 		html_engine_edit_selection_updater_reset (e->selection_updater);
 		e->selection = NULL;
+		/*
+		if (gdk_selection_owner_get (GDK_SELECTION_PRIMARY) == GTK_WIDGET (e->widget)->window)
+			gtk_selection_owner_set (NULL, GDK_SELECTION_PRIMARY, 
+						 html_selection_current_time ());    
+		*/
 	}
 }
 
@@ -118,6 +137,7 @@ html_engine_disable_selection (HTMLEngine *e)
 
 	remove_mark (e);
 	html_engine_unselect_all (e);
+	e->selection_mode = FALSE;
 }
 
 static gboolean
@@ -126,14 +146,21 @@ line_interval (HTMLEngine *e, HTMLCursor *begin, HTMLCursor *end)
 	return html_cursor_beginning_of_line (begin, e) && html_cursor_end_of_line (end, e);
 }
 
+gboolean
+html_selection_word (gunichar uc)
+{
+	return uc && uc != ' ' && uc != '\t' && uc != ENTITY_NBSP         /* white space */
+		&& uc != '(' && uc != ')' && uc != '[' && uc != ']';
+}
+
 static gboolean
 word_interval (HTMLEngine *e, HTMLCursor *begin, HTMLCursor *end)
 {
 	/* move to the begin of word */
-	while (html_is_in_word (html_cursor_get_prev_char (begin)))
+	while (html_selection_word (html_cursor_get_prev_char (begin)))
 		html_cursor_backward (begin, e);
 	/* move to the end of word */
-	while (html_is_in_word (html_cursor_get_current_char (end)))
+	while (html_selection_word (html_cursor_get_current_char (end)))
 		html_cursor_forward (end, e);
 
 	return (begin->object && end->object);
@@ -182,9 +209,55 @@ html_engine_is_selection_active (HTMLEngine *e)
 	return e->selection ? TRUE : FALSE;
 }
 
+static void
+test_point (HTMLObject *o, HTMLEngine *e, gpointer data)
+{
+	HTMLPoint *point = (HTMLPoint *) data;
+
+	if (point->object == o) {
+		if (point->object == e->selection->from.object && point->offset < e->selection->from.offset)
+			return;
+		if (point->object == e->selection->to.object && point->offset > e->selection->to.offset)
+			return;
+
+		/* this indicates that object is IN the selection */
+		point->object = NULL;
+	}
+}
+
+gboolean
+html_engine_point_in_selection (HTMLEngine *e, HTMLObject *obj, guint offset)
+{
+	HTMLPoint *point;
+	gboolean rv;
+
+	if (!html_engine_is_selection_active (e) || !obj)
+		return FALSE;
+
+	point = html_point_new (obj, offset);
+	html_interval_forall (e->selection, e, test_point, point);
+	rv = point->object == NULL;
+
+	html_point_destroy (point);
+
+	return rv;
+}
+
 void
 html_engine_activate_selection (HTMLEngine *e, guint32 time)
 {
-	if (e->selection && GTK_WIDGET_REALIZED (e->widget))
+	if (e->selection && e->block_selection == 0 && GTK_WIDGET_REALIZED (e->widget))
 		gtk_selection_owner_set (GTK_WIDGET (e->widget), GDK_SELECTION_PRIMARY, time);	
 }
+
+void
+html_engine_block_selection (HTMLEngine *e)
+{
+	e->block_selection ++;
+}
+
+void
+html_engine_unblock_selection (HTMLEngine *e)
+{
+	e->block_selection --;
+}
