@@ -94,22 +94,27 @@ get_tags (const HTMLText *text,
 	size = font_style & GTK_HTML_FONT_STYLE_SIZE_MASK;
 	if (size != 0) {
 		opening_p += sprintf (opening_p, "<FONT SIZE=\"%d\">", size);
+		ending_p += sprintf (ending_p, "</FONT>");
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_BOLD) {
 		opening_p += sprintf (opening_p, "<B>");
+		ending_p += sprintf (ending_p, "</B>");
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_ITALIC) {
 		opening_p += sprintf (opening_p, "<I>");
+		ending_p += sprintf (ending_p, "</I>");
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_UNDERLINE) {
 		opening_p += sprintf (opening_p, "<U>");
+		ending_p += sprintf (ending_p, "</U>");
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_STRIKEOUT) {
 		opening_p += sprintf (opening_p, "<S>");
+		ending_p += sprintf (ending_p, "</S>");
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_FIXED) {
@@ -117,27 +122,6 @@ get_tags (const HTMLText *text,
 		ending_p += sprintf (ending_p, "</TT>");
 	}
 
-	if (font_style & GTK_HTML_FONT_STYLE_STRIKEOUT) {
-		ending_p += sprintf (ending_p, "</S>");
-	}
-
-	if (font_style & GTK_HTML_FONT_STYLE_UNDERLINE) {
-		ending_p += sprintf (ending_p, "</U>");
-	}
-
-	if (font_style & GTK_HTML_FONT_STYLE_ITALIC) {
-		ending_p += sprintf (ending_p, "</I>");
-	}
-
-	if (font_style & GTK_HTML_FONT_STYLE_BOLD) {
-		ending_p += sprintf (ending_p, "</B>");
-	}
-
-	if (size != 0) {
-		ending_p += sprintf (ending_p, "</FONT SIZE=\"%d\">", size);
-	}
-
-
 	*opening_p = 0;
 	*ending_p = 0;
 }
@@ -282,6 +266,9 @@ html_text_op_cut_helper (HTMLText *text, HTMLEngine *e, GList *from, GList *to,
 	} else {
 		text->spell_errors = remove_spell_errors (text->spell_errors, 0, text->text_len);
 		html_object_move_cursor_before_remove (HTML_OBJECT (text), e);
+		html_object_change_set (HTML_OBJECT (text)->parent, HTML_CHANGE_ALL_CALC);
+		/* force parent redraw */
+		HTML_OBJECT (text)->parent->width = 0;
 		html_object_remove_child (HTML_OBJECT (text)->parent, HTML_OBJECT (text));
 
 		rv    = HTML_OBJECT (text);
@@ -317,7 +304,7 @@ op_cut (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left, GL
 }
 
 static gboolean
-object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GList *right)
+object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList **left, GList **right, HTMLCursor *cursor)
 {
 	HTMLText *t1, *t2;
 	gchar *to_free;
@@ -332,6 +319,11 @@ object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GL
 
 	/* merge_word_width (t1, t2, e->painter); */
 
+	if (e->cursor->object == with) {
+		e->cursor->object  = self;
+		e->cursor->offset += t1->text_len;
+	}
+
 	move_spell_errors (t2->spell_errors, 0, t1->text_len);
 	t1->spell_errors = g_list_concat (t1->spell_errors, t2->spell_errors);
 	t2->spell_errors = NULL;
@@ -383,11 +375,11 @@ object_split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, g
 	html_clue_append_after (HTML_CLUE (self->parent), dup, self);
 
 	prev = self->prev;
-	if (t1->text_len == 0 && prev && html_object_merge (prev, self, e, NULL, NULL))
+	if (t1->text_len == 0 && prev && html_object_merge (prev, self, e, NULL, NULL, NULL))
 		self = prev;
 
 	if (t2->text_len == 0 && dup->next)
-		html_object_merge (dup, dup->next, e, NULL, NULL);
+		html_object_merge (dup, dup->next, e, NULL, NULL, NULL);
 
 	/* printf ("--- before split offset %d dup len %d\n", offset, HTML_TEXT (dup)->text_len);
 	   debug_spell_errors (HTML_TEXT (self)->spell_errors); */
@@ -420,8 +412,7 @@ object_split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, g
 }
 
 static gboolean
-calc_size (HTMLObject *self,
-	   HTMLPainter *painter)
+calc_size (HTMLObject *self, HTMLPainter *painter, GList **changed_objs)
 {
 	HTMLText *text = HTML_TEXT (self);
 	GtkHTMLFontStyle style = html_text_get_font_style (text);
@@ -586,7 +577,7 @@ forward_get_nb_width (HTMLText *text, HTMLPainter *painter, gboolean begin)
 	g_assert (text->text_len == 0);
 
 	/* find prev/next object */
-	obj = (begin)
+	obj = begin
 		? html_object_prev_not_slave (HTML_OBJECT (text))
 		: html_object_next_not_slave (HTML_OBJECT (text));
 
@@ -597,6 +588,27 @@ forward_get_nb_width (HTMLText *text, HTMLPainter *painter, gboolean begin)
 		return html_text_get_nb_width (HTML_TEXT (obj), painter, begin);
 }
 
+static gint
+get_next_nb_width (HTMLText *text, HTMLPainter *painter, gboolean begin)
+{
+	HTMLObject *obj;
+
+	g_assert (text);
+	g_assert (html_object_is_text (HTML_OBJECT (text)));
+	g_assert (text->words == 1);
+
+	/* find prev/next object */
+	obj = begin
+		? html_object_next_not_slave (HTML_OBJECT (text))
+		: html_object_prev_not_slave (HTML_OBJECT (text));
+
+	/* if not found or not text return 0, otherwise forward get_nb_with there */
+	if (!obj || !html_object_is_text (obj))
+		return 0;
+	else
+		return html_text_get_nb_width (HTML_TEXT (obj), painter, begin);
+}
+
 static inline guint
 word_width (HTMLText *text, HTMLPainter *p, guint i)
 {
@@ -622,7 +634,8 @@ html_text_get_nb_width (HTMLText *text, HTMLPainter *painter, gboolean begin)
 
 	html_text_request_word_width (text, painter);
 
-	return word_width (text, painter, begin ? 0 : text->words - 1);
+	return word_width (text, painter, begin ? 0 : text->words - 1)
+		+ (text->words == 1 ? get_next_nb_width (text, painter, begin) : 0);
 }
 
 static gint
@@ -999,7 +1012,6 @@ set_color (HTMLText *text,
 	text->color = color;
 
 	if (engine != NULL) {
-		html_object_relayout (HTML_OBJECT (text)->parent, engine, HTML_OBJECT (text));
 		html_engine_queue_draw (engine, HTML_OBJECT (text));
 	}
 }
@@ -1133,9 +1145,15 @@ get_cursor (HTMLObject *self,
 	    gint *x1, gint *y1,
 	    gint *x2, gint *y2)
 {
-	HTMLObject *slave;
+	HTMLObject *slave, *next;
 	guint ascent, descent;
 
+	next = html_object_next_not_slave (self);
+	if (offset == HTML_TEXT (self)->text_len && next && html_object_is_text (next)) {
+		html_object_get_cursor (next, painter, 0, x1, y1, x2, y2);
+		return;
+	}
+
 	html_object_get_cursor_base (self, painter, offset, x2, y2);
 
 	slave = self->next;
@@ -1158,7 +1176,13 @@ get_cursor_base (HTMLObject *self,
 		 guint offset,
 		 gint *x, gint *y)
 {
-	HTMLObject *obj;
+	HTMLObject *obj, *next;
+
+	next = html_object_next_not_slave (self);
+	if (offset == HTML_TEXT (self)->text_len && next && html_object_is_text (next)) {
+		html_object_get_cursor_base (next, painter, 0, x, y);
+		return;
+	}
 
 	for (obj = self->next; obj != NULL; obj = obj->next) {
 		HTMLTextSlave *slave;
@@ -1371,6 +1395,7 @@ html_text_set_text (HTMLText *text, const gchar *new_text)
 {
 	g_free (text->text);
 	text->text = g_strdup (new_text);
+	text->text_len = text_len (new_text, -1);
 	html_object_change_set (HTML_OBJECT (text), HTML_CHANGE_ALL);
 }
 
@@ -1566,6 +1591,8 @@ html_text_magic_link (HTMLText *text, HTMLEngine *engine, guint offset)
 		return FALSE;
 	offset--;
 
+	/* printf ("html_text_magic_link\n"); */
+
 	html_undo_level_begin (engine->undo, "Magic link", "Remove magic link");
 	saved_position = engine->cursor->position;
 
